import click
import openai

from skainet import utils
from skainet.data import CONFIG

DEFAULT_CHAT_MODEL = CONFIG["moderation"]["model"]


@click.command()
@click.argument("input", type=utils.Prompt(), default="")
@click.option(
    "-m",
    "--model",
    type=str,
    default=DEFAULT_CHAT_MODEL,
    help=f"Model selection. Default is {DEFAULT_CHAT_MODEL}.",
)
def moderate(input: str, model: str):
    """Check if text violates OpenAI's Content Policy"""
    try:
        response = openai.Moderation.create(
            input=input,
            model=model,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        print(response)



import json
import os
import platform
import sys
from configparser import ConfigParser
from pathlib import Path
from typing import Dict, List

## Data Directory
if platform.system() == "Windows":
    appdata = os.getenv("LOCALAPPDATA")
    DATA_DIR = Path(appdata) / __package__
else:
    DATA_DIR = Path.home() / ".local" / "share" / __package__

try:
    DATA_DIR.mkdir(parents=True, exist_ok=True)
except OSError as e:
    print(f"Error while attempting to create data directory: {e}")
    sys.exit(1)


## Key File
_KEY_FILE = DATA_DIR / "api_key"
if not _KEY_FILE.exists():
    _KEY_FILE.touch()


def load_key() -> str:
    return _KEY_FILE.read_text()


def save_key(key: str):
    _KEY_FILE.write_text(key)


## Chat History
_CHAT_FILE = DATA_DIR / "chat_history.json"
if not _CHAT_FILE.exists():
    _CHAT_FILE.write_text("[]")


def load_chat() -> List[Dict[str, str]]:
    with open(_CHAT_FILE) as file:
        chat = json.load(file)
    return chat


def save_chat(chat: List[Dict[str, str]]):
    with open(_CHAT_FILE, "w") as file:
        json.dump(chat, file, indent=2)


# Configuration
default_config_path = Path(__file__).parent / "config.ini"
if not default_config_path.exists():
    print(f"Default config ({default_config_path}) does not exist!")
    sys.exit(1)

DEFAULT_CONFIG = ConfigParser()
DEFAULT_CONFIG.read(default_config_path)


# Create config file, copying default config if it doesn't exist
_CONFIG_FILE = DATA_DIR / "config.ini"
if not _CONFIG_FILE.exists():
    if platform.system() == "Darwin":
        DEFAULT_CONFIG["general"]["editor"] = "open"
    elif platform.system() == "Windows":
        DEFAULT_CONFIG["general"]["editor"] = "notepad"
        editor = "notepad.exe"
    else:
        DEFAULT_CONFIG["general"]["editor"] = "nano"

    with open(_CONFIG_FILE, "w") as configfile:
        DEFAULT_CONFIG.write(configfile)

CONFIG = ConfigParser()
CONFIG.read(_CONFIG_FILE)


# Add any settings in default_config that don't exist in config file
for section in DEFAULT_CONFIG.sections():
    if section not in CONFIG:
        CONFIG.add_section(section)

    for key, value in DEFAULT_CONFIG.items(section):
        if key not in CONFIG[section]:
            CONFIG[section][key] = value

with open(_CONFIG_FILE, "w") as configfile:
    CONFIG.write(configfile)



import functools
import sys
from typing import Dict, List

import click
import openai

from skainet import utils
from skainet.data import CONFIG, load_chat, save_chat


def calculate_tokens(string: str) -> int:
    return len(string.split())


def calculate_context(message_list: List[Dict[str, str]]):
    return sum([calculate_tokens(msg["content"]) for msg in message_list])


def truncate_context(
    message_list: List[Dict[str, str]], context_limit: int
) -> List[Dict[str, str]]:
    context = []
    for message in reversed(message_list):
        new_tokens = calculate_tokens(message["content"])
        if calculate_context(context) + new_tokens < context_limit:
            context.insert(0, message)
        else:
            break

    return context


def text_options(model, num, temp):
    def decorator(function):
        @click.option(
            "-m",
            "--model",
            type=str,
            default=model,
            help=f"Model selection",
        )
        @click.option(
            "-n",
            "--num",
            type=click.IntRange(min=1),
            default=num,
            help=f"Number of completions to generate",
        )
        @click.option(
            "-t",
            "--temp",
            type=click.FloatRange(min=0, max=2),
            default=temp,
            help=f"Sampling temperature selection",
        )
        @functools.wraps(function)
        def wrapper_common_options(*args, **kwargs):
            return function(*args, **kwargs)

        return wrapper_common_options

    return decorator


MAXIMUM_CONTEXT = 4096
SYSTEM_MESSAGE = {
    "role": "system",
    "content": CONFIG["chat"]["seed_prompt"],
}
SYSTEM_MESSAGE_LEN = calculate_tokens(SYSTEM_MESSAGE["content"])
DEFAULT_CHAT_CONTEXT = int(CONFIG["chat"]["context"])
DEFAULT_CHAT_MODEL = CONFIG["chat"]["model"]
DEFAULT_CHAT_TEMPERATURE = int(CONFIG["chat"]["temperature"])
DEFAULT_CHAT_MAX_TOKENS = int(CONFIG["chat"]["max_tokens"])
DEFAULT_CHAT_NUM = int(CONFIG["chat"]["num"])


@click.command(context_settings={"show_default": True})
@click.argument("prompt", type=utils.Prompt(), default="")
@text_options(DEFAULT_CHAT_MODEL, DEFAULT_CHAT_NUM, DEFAULT_CHAT_TEMPERATURE)
@click.option(
    "-c",
    "--context",
    type=click.IntRange(min=1, max=MAXIMUM_CONTEXT - SYSTEM_MESSAGE_LEN),
    default=DEFAULT_CHAT_CONTEXT,
    help=f"Context length, for chat",
)
@click.option("-ch", "--clearhistory", is_flag=True, help="Clear entire chat history")
@click.option(
    "-mt",
    "--maxtokens",
    type=int,
    default=DEFAULT_CHAT_MAX_TOKENS,
    help=f"Maximum number of tokens in response, set to -1 to remove limit",
)
@click.option(
    "-s",
    "--stop",
    type=str,
    multiple=True,
    help=f"Sequences where the API will stop generating further tokens. Up to four are allowed. The returned text will not contain the stop sequence",
)
@click.option("-ns", "--no-stream", is_flag=True, help=f"Disable response streaming")
@click.option("-nu", "--no-update", is_flag=True, help=f"Do not update chat history")
def chat(
    prompt: str,
    model: str,
    num: int,
    temp: float,
    context: int,
    clearhistory: bool,
    maxtokens: int,
    stop: int,
    no_stream: bool,
    no_update: bool,
):
    """Chat with ChatGPT

    Send PROMPT to ChatGPT. PROMPT can be a string, filepath, or piped in.
    If no PROMPT is given, Skai will open $EDITOR or your configured text editor.
    """

    if clearhistory is True:
        save_chat([])

    if maxtokens < 0:
        maxtokens = None

    if not stop:
        stop = None

    # Load chat history and append new prompt
    chat_history = load_chat()
    new_prompt = {"role": "user", "content": prompt}
    chat_history.append(new_prompt)

    # Limit chat context & instert seed prompt
    available_context = context - SYSTEM_MESSAGE_LEN
    current_context = truncate_context(chat_history, available_context)
    if current_context != chat_history:
        print(
            "Warning: your chat history has been truncated to fit the context limit",
            file=sys.stderr,
        )
    current_context.insert(0, SYSTEM_MESSAGE)

    # Send request
    try:
        response = openai.ChatCompletion.create(
            model=model,
            messages=current_context,
            temperature=temp,
            # top_p=args.top_p,
            n=num,
            stream=not no_stream,
            stop=stop,
            max_tokens=maxtokens,
            # presence_penalty=args.presence_penalty,
            # frequency_penalty=args.frequency_penalty,
            # user=args.user
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        if num > 1:
            if no_stream:
                for index, choice in enumerate(response["choices"]):
                    content = choice["message"]["content"]
                    print(f"({index}) {content}")
            else:
                messages = {}
                for chunk in response:
                    for choice in chunk["choices"]:
                        index = choice["index"]
                        if index not in messages:
                            messages[index] = ""

                        delta = choice["delta"]
                        if "content" in delta:
                            content = delta["content"]
                            messages[index] += content

                for index in range(len(messages)):
                    print(f"({index}) {messages[index]}")
        else:
            if no_stream:
                new_response = response["choices"][0]["message"]
                print(new_response["content"])
            else:
                new_response = {"role": "", "content": ""}
                for chunk in response:
                    # Parse response chunk
                    choice = chunk["choices"][0]
                    delta = choice["delta"]
                    if "role" in delta:
                        new_response["role"] = delta["role"]
                        text = ""
                    elif "content" in delta:
                        new_response["content"] += delta["content"]
                        text = delta["content"]
                    elif not delta:
                        text = "\n"
                    else:
                        text = ""

                    print(text, end="")

            if not no_update:
                chat_history.append(new_response)
                chat_history = truncate_context(chat_history, MAXIMUM_CONTEXT)
                save_chat(chat_history)


DEFAULT_COMPLETE_MODEL = CONFIG["completion"]["model"]
DEFAULT_COMPLETE_SUFFIX = CONFIG["completion"]["suffix"]
DEFAULT_COMPLETE_MAX_TOKENS = int(CONFIG["completion"]["max_tokens"])
DEFAULT_COMPLETE_TEMPERATURE = int(CONFIG["completion"]["temperature"])
DEFAULT_COMPLETE_NUM = int(CONFIG["completion"]["num"])


@click.command(context_settings={"show_default": True})
@click.argument("prompt", type=utils.Prompt(), default="")
@text_options(
    DEFAULT_COMPLETE_MODEL, DEFAULT_COMPLETE_NUM, DEFAULT_COMPLETE_TEMPERATURE
)
@click.option(
    "-mt",
    "--maxtokens",
    type=int,
    default=DEFAULT_COMPLETE_MAX_TOKENS,
    help=f"Maximum number of tokens in response, set to -1 to remove limit",
)
@click.option(
    "-s",
    "--stop",
    type=str,
    multiple=True,
    help=f"Sequences where the API will stop generating further tokens. Up to four are allowed. The returned text will not contain the stop sequence.",
)
@click.option("-ns", "--no-stream", is_flag=True, help=f"Disable response streaming.")
@click.option(
    "-sf",
    "--suffix",
    type=str,
    default=DEFAULT_COMPLETE_SUFFIX,
    help=f"Suffix that comes after a completion of inserted text.",
)
@click.option(
    "--echo", is_flag=True, help=f"Echo back the prompt in addition to the completion"
)
def complete(
    prompt: str,
    model: str,
    suffix: str,
    maxtokens: int,
    temp: float,
    num: int,
    echo: bool,
    stop: List[str],
    no_stream: bool,
):
    """Text Completion

    Return a completion for a given prompt. PROMPT can be a string, filepath, or piped in.
    If no PROMPT is given, Skai will open $EDITOR or your configured text editor.
    """

    if maxtokens < 0:
        maxtokens = None

    if not suffix:
        suffix = None

    if not stop:
        stop = None

    # Send request
    try:
        response = openai.Completion.create(
            model=model,
            prompt=prompt,
            suffix=suffix,
            max_tokens=maxtokens,
            temperature=temp,
            # top_p=args.top_p,
            n=num,
            stream=not no_stream,
            # logprobs=args.logprobs,
            echo=echo,
            stop=stop,
            # presence_panalty=args.presence_penalty,
            # frequency_penalty=args.frequency_penalty,
            # best_of=args.best_of,
            # logit_bias=args.logit_bias,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        if num > 1:
            if no_stream:
                for index, choice in enumerate(response["choices"]):
                    text = choice["text"]
                    print(f"({index}) {text}")
            else:
                messages = {}
                for chunk in response:
                    for choice in chunk["choices"]:
                        index = choice["index"]
                        if index not in messages:
                            messages[index] = ""

                        text = choice["text"]
                        messages[index] += text

                for index, text in messages.items():
                    print(f"({index}) {text}")
        else:
            if no_stream:
                text = response["choices"][0]["text"]
                print(text, end="")
            else:
                for chunk in response:
                    text = chunk["choices"][0]["text"]
                    print(text, end="")


DEFAULT_EDIT_MODEL = CONFIG["edit"]["model"]
DEFAULT_EDIT_TEMPERATURE = int(CONFIG["edit"]["temperature"])
DEFAULT_EDIT_NUM = int(CONFIG["edit"]["num"])


@click.command(context_settings={"show_default": True})
@click.argument("instruction", type=str)
@click.argument("input", type=utils.Prompt(), default="")
@text_options(DEFAULT_EDIT_MODEL, DEFAULT_EDIT_NUM, DEFAULT_EDIT_TEMPERATURE)
def edit(instruction: str, input: str, model: str, temp: float, num: int):
    """Text editing

    Edit a given input according to the instruction. INPUT a string, filepath, or piped in.
    If no INPUT is given, Skai will open $EDITOR or your configured text editor.
    """

    # Send request
    try:
        response = openai.Edit.create(
            model=model,
            input=input,
            instruction=instruction,
            n=num,
            temperature=temp,
            # top_p=args.top_p,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        if num > 1:
            for choice in response["choices"]:
                print(f"({choice['index']}) {choice['text']}")
        else:
            text = response["choices"][0]["text"]
            print(text)



import sys

import click
import openai

from skainet import utils


@click.group("model", help="Get information about available models")
def model():
    """DOCUMENTATION"""
    pass


@model.command()
def list():
    """List available models"""
    try:
        model_list = openai.Model.list()["data"]
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        print("Available Models:")
        model_names = [model["id"] for model in model_list]
        for name in sorted(model_names):
            print(f"{name}")


@model.command()
@click.argument("model_name", type=str)
def show(model_name: str):
    """Get information about a model"""
    try:
        model_list = openai.Model.list()["data"]
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        model_info = {}
        for model in model_list:
            if model["id"] == model_name:
                model_info = model

        if model_info:
            print(model_info)
        else:
            print(f"No model with name '{model_name}' found.")
            sys.exit(1)



import sys
from typing import List

import click

from skainet.data import _CONFIG_FILE, CONFIG


@click.group("config", help="Skainet configuration")
def config():
    """DOCUMENTATION"""
    pass


@config.command()
def show():
    """Display contents of config file"""
    print(_CONFIG_FILE.read_text())


@config.command()
def path():
    """Show path to config file"""
    print(str(_CONFIG_FILE.absolute()))


@config.command()
@click.argument("setting", nargs=-1)
@click.argument("value", type=str)
def set(setting: List[str], value: str):
    """Set a variable in the config file"""
    config_path = setting
    try:
        dic = CONFIG
        for key in config_path[:-1]:
            dic = dic[key]
        dic[config_path[-1]] = value
    except KeyError as e:
        print(f"{' '.join(setting)} does not exist")
        sys.exit(1)

    with open(_CONFIG_FILE, "w") as configfile:
        CONFIG.write(configfile)



import os

import click
import openai
from importlib_metadata import version

from skainet.audio import audio
from skainet.config import config
from skainet.data import load_key, save_key
from skainet.file import file
from skainet.image import image
from skainet.model import model
from skainet.moderate import moderate
from skainet.text import chat, complete, edit


@click.group(invoke_without_command=True)
@click.version_option(version("skainet"))
@click.option("--key", "new_key", is_flag=True, help="Show prompt to save an API key")
def main(new_key: bool):
    """Skainet - Shell tool for interacting with the OpenAI API"""

    if "OPENAI_API_KEY" in os.environ:
        key = os.environ["OPENAI_API_KEY"]
    else:
        key = load_key()

    if new_key or not key:
        key = input(
            "Please enter your API key\nTo generate an API key, see:\nhttps://platform.openai.com/account/api-keys\n:"
        )
        save_key(key)

    openai.api_key = key


main.add_command(chat)
main.add_command(edit)
main.add_command(complete)
main.add_command(config)
main.add_command(image)
main.add_command(model)
main.add_command(moderate)
main.add_command(file)
main.add_command(audio)


if __name__ == "__main__":
    main()



import base64
import functools
import os
import platform
import sys
import tempfile
import uuid
import webbrowser
from pathlib import Path
from typing import BinaryIO

import click
import openai

from skainet import utils
from skainet.data import CONFIG

DEFAULT_NUM = int(CONFIG["image"]["num"])
DEFAULT_SIZE = CONFIG["image"]["size"]
DEFAULT_FORMAT = CONFIG["image"]["format"]

URL_CHOICES = [
    "url",
    "b64_json",
]

SIZE_CHOICES = [
    "1024x1024",
    "512x512",
    "256x256",
]


@click.group("image", help="Image generation and manipulation")
def image():
    """DOCUMENTATION"""
    pass


def image_options(function):
    @click.option(
        "-n",
        "--num",
        type=click.IntRange(min=1),
        default=DEFAULT_NUM,
        help=f"Number of images to generate. Default is {DEFAULT_NUM}",
    )
    @click.option(
        "-s",
        "--size",
        type=click.Choice(SIZE_CHOICES),
        default=DEFAULT_SIZE,
        help=f"Image size",
    )
    @click.option(
        "-fmt",
        "--format",
        type=click.Choice(URL_CHOICES),
        default=DEFAULT_FORMAT,
        help=f"Format of returned images",
    )
    @click.option(
        "-p",
        "--print",
        "print_response",
        is_flag=True,
        help="Output response to terminal instead of opening",
    )
    @functools.wraps(function)
    def wrapper_common_options(*args, **kwargs):
        return function(*args, **kwargs)

    return wrapper_common_options


@image.command(context_settings={"show_default": True})
@click.argument("prompt", type=utils.Prompt(), default="")
@image_options
def create(prompt: str, num: int, size: str, format: str, print_response: bool):
    """Image generation

    Generate an image from a PROMPT. PROMPT can be a string, filepath, or piped in.
    If no PROMPT is given, Skai will open $EDITOR or your configured text editor.
    """

    try:
        response = openai.Image.create(
            prompt=prompt,
            n=num,
            size=size,
            response_format=format,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        display_image_response(response, format, num, print_response)


@image.command(context_settings={"show_default": True})
@click.argument("image", type=utils.File("rb", exts=[".png"]))
@click.argument("mask", type=utils.File("rb", exts=[".png"]))
@click.argument("prompt", type=utils.Prompt(), default="")
@image_options
def edit(
    image: BinaryIO,
    mask: BinaryIO,
    prompt: str,
    num: int,
    size: str,
    format: str,
    print_response: bool,
):
    """Edit an image.

    Generate an edit from a PROMPT. PROMPT can be a string, filepath, or piped in.
    If no PROMPT is given, Skai will open $EDITOR or your configured text editor.

    IMAGE image to be editted
    MASK image with transparent areas to indicate where image should be editted
    """

    try:
        response = openai.Image.create_edit(
            image=image,
            mask=mask,
            prompt=prompt,
            n=num,
            size=size,
            response_format=format,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        display_image_response(response, format, num, print_response)


@image.command(
    help="Create a variation of an image", context_settings={"show_default": True}
)
@click.argument("image", type=utils.File("rb", exts=[".png"]))
@image_options
def variation(
    image: BinaryIO,
    num: int,
    size: str,
    format: str,
    print_response: bool,
):
    """DOCUMENTATION"""
    input_image = image.read()

    try:
        response = openai.Image.create_variation(
            image=input_image,
            n=num,
            size=size,
            response_format=format,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        display_image_response(response, format, num, print_response)


def display_image_response(response, format, num, print_response):
    for index, generation in enumerate(response["data"]):
        if format == "url":
            image = generation[format]
        elif format == "b64_json":
            b64_encoded_data = generation[format]
            decoded_image = base64.b64decode(b64_encoded_data)
            temp_file = Path(tempfile.gettempdir()) / str(uuid.uuid4())
            temp_file = temp_file.with_suffix(".png")
            temp_file.write_bytes(decoded_image)
            image = temp_file

        if print_response is True:
            if num > 1:
                print(f"({index}): {image}")
            else:
                print(f"{image}")
        else:
            try:
                if format == "b64_json":
                    image = image.as_uri()

                webbrowser.open(image)
            except Exception as e:
                print(f"Error opening image: {e}")
                sys.exit(1)



import itertools
from pathlib import Path

import click
import openai

from skainet import utils

FILE_PURPOSES = [
    "fine-tune",
]


@click.group(help="File management")
def file():
    pass


@file.command()
def list():
    """List files"""
    try:
        response = openai.File.list()
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        if response["data"]:
            print("Files:")
            for file in response["data"]:
                print(file)
        else:
            print("No files")


@file.command()
@click.argument("file", type=click.Path(exists=True, path_type=Path))
@click.option(
    "-p",
    "--purpose",
    type=click.Choice(FILE_PURPOSES),
    default="fine-tune",
    help="Purpose of file",
)
def upload(
    file: Path,
    purpose: str,
):
    """Upload file"""
    try:
        response = openai.File.create(
            file=file.read_text(), purpose=purpose, user_provided_filename=file.name
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        print(response)


@file.command()
@click.argument("file_id", metavar="ID", type=str)
def delete(
    file_id: str,
):
    """Delete file"""
    try:
        openai.File.delete(
            sid=file_id,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        pass


@file.command()
@click.argument("file_id", metavar="ID", type=str)
@click.option(
    "-o",
    "--output",
    type=click.Path(writable=True, path_type=Path),
    help="Override output file destination",
)
def download(file_id: str, output: Path):
    """Download file"""
    try:
        response_bytes = openai.File.download(
            id=file_id,
        )

        if not output or output.is_dir():
            file_list = openai.File.list()
            for file_info in file_list["data"]:
                if file_info["id"] == file_id:
                    file_name = file_info["filename"]

            if not output:
                output = Path.cwd() / file_name
            else:
                output = output / file_name
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        if output.exists():
            for num in itertools.count(start=1):
                test_file = output.parent / (output.stem + f"{num}" + output.suffix)
                if not test_file.exists():
                    output = test_file
                    break

        output.write_bytes(response_bytes)


@file.command()
@click.argument("name", type=str)
@click.argument("size", type=int)
@click.argument("purpose", type=click.Choice(FILE_PURPOSES), default="fine-tune")
def find(
    name: str,
    size: int,
    purpose: str,
):
    """File information"""
    try:
        response = openai.File.find_matching_files(
            name=name,
            bytes=size,
            purpose=purpose,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        for file in response:
            print(file)



import functools
import os
import subprocess
import sys
import tempfile
import uuid
from pathlib import Path
from typing import Any, Callable, List, Optional

import click
import openai

from skainet.data import CONFIG


def create_tempfile(ext: str) -> Path:
    temp_file = Path(tempfile.gettempdir()) / str(uuid.uuid4())
    temp_file = temp_file.with_suffix(ext)
    temp_file.touch()
    return temp_file


def open_editor():
    temp_file = create_tempfile(".txt")

    if "EDITOR" in os.environ:
        editor = os.environ["EDITOR"]
    else:
        editor = CONFIG["general"]["editor"]

    subprocess.run([editor, str(temp_file)], check=True)

    return temp_file.read_text()


def handle_error(error: openai.OpenAIError):
    print(f"{error.__class__.__name__}: {error}")
    sys.exit(1)


class File(click.File):
    """
    Adding extra arguments to the click.File type to support file extension validation
    """

    def __init__(self, *args, exts: List[str], **kwargs):
        self.valid_ext = exts
        super().__init__(*args, **kwargs)

    def convert(self, value, param, ctx):
        file = super().convert(value, param, ctx)
        if self.valid_ext:
            extention = Path(file.name).suffix
            if extention not in self.valid_ext:
                self.fail(
                    f"{extention} is not a supported file type for this command",
                    param,
                    ctx,
                )
        return file


class Prompt(click.ParamType):
    name = "prompt"

    def convert(self, value: str, param, ctx):
        if not sys.stdin.isatty():
            if value:
                value = value + "\n"
            value += sys.stdin.read()

        if not value:
            value = open_editor()

        return value



import functools
from typing import BinaryIO

import click
import openai

from skainet import utils
from skainet.data import CONFIG

DEFAULT_AUDIO_MODEL = CONFIG["audio"]["model"]
DEFAULT_AUDIO_TEMPERATURE = int(CONFIG["audio"]["temperature"])
DEFAULT_AUDIO_RESPONSE_FORMAT = CONFIG["audio"]["format"]
DEFAULT_AUDIO_LANGUAGE = CONFIG["audio"]["language"]

RESPONSE_FORMAT_CHOICES = [
    "json",
    "text",
    "srt",
    "verbose_json",
    "vtt",
]

FILE_FORMAT_CHOICES = [
    "mp3",
    "mp4",
    "mpeg",
    "mpga",
    "m4a",
    "wav",
    "webm",
]


@click.group(help="Audio translation and transcription")
def audio():
    pass


def audio_options():
    def decorator(function):
        @click.argument(
            "audio",
            type=utils.File(
                "rb",
                exts=[
                    ".mp3",
                    ".mp4",
                    ".mpeg",
                    ".m4a",
                    ".wav",
                    ".webm",
                ],
            ),
        )
        @click.option(
            "-p", "--prompt", type=utils.Prompt(), help="Prompt to help guide model"
        )
        @click.option(
            "-m",
            "--model",
            type=str,
            default=DEFAULT_AUDIO_MODEL,
            help=f"Model selection",
        )
        @click.option(
            "-t",
            "--temp",
            type=click.FloatRange(min=0, max=2),
            default=DEFAULT_AUDIO_TEMPERATURE,
            help=f"Sampling temperature selection",
        )
        @click.option(
            "-fmt",
            "--format",
            type=click.Choice(RESPONSE_FORMAT_CHOICES),
            default=DEFAULT_AUDIO_RESPONSE_FORMAT,
            help=f"The format of the transcript output. Default is {DEFAULT_AUDIO_RESPONSE_FORMAT}",
        )
        @functools.wraps(function)
        def wrapper_common_options(*args, **kwargs):
            return function(*args, **kwargs)

        return wrapper_common_options

    return decorator


@audio.command(context_settings={"show_default": True})
@audio_options()
@click.option(
    "-l",
    "--lang",
    type=str,
    default=DEFAULT_AUDIO_LANGUAGE,
    help=f"Language of the input audio. Supplying the input language in ISO-639-1 format will improve accuracy and latency",
)
def transcribe(
    audio: BinaryIO,
    prompt: str,
    model: str,
    temp: float,
    format: str,
    lang: str,
):
    """Transcribes audio into the input language"""

    try:
        response = openai.Audio.transcribe(
            model=model,
            file=audio,
            temperature=temp,
            format=format,
            prompt=prompt,
            language=lang,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        print(response["text"])


@audio.command(context_settings={"show_default": True})
@audio_options()
def translate(
    audio: BinaryIO,
    prompt: str,
    model: str,
    temp: float,
    format: str,
):
    """Translate audio into English"""

    try:
        response = openai.Audio.translate(
            model=model,
            file=audio,
            temperature=temp,
            format=format,
            prompt=prompt,
        )
    except openai.OpenAIError as e:
        utils.handle_error(e)
    else:
        print(response["text"])
